/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <sstream>

#include "idlc/generator.h"
#include "idlc/block.h"
#include "idlc/interface.h"
#include "idlc/structure.h"
#include "idlc/declaration.h"
#include "idlc/attribute.h"

namespace tidl {

Generator::Generator(std::shared_ptr<Document> doc) : doc_(doc) {}

void Generator::Run(const std::string& file_name) {
  FileName = file_name;
  out_file_.open(FileName);

  OnInitGen(out_file_);
  OnFiniGen(out_file_);
  out_file_.close();
}

std::string Generator::AddIndent(int indent, std::string lines, bool space) {
  std::stringstream ss(lines);
  std::string result;
  std::string to;

  while (std::getline(ss, to, '\n')) {
    if (to.length() > 0) {
      for (int i = 0; i < indent; i++) {
        if (space)
          result += " ";
        else
          result += "\t";
      }
    }

    result += to;
    result += "\n";
  }

  return result;
}

std::string Generator::GetFileNamespace() const {
  std::string key1(".");
  std::string key2("/");

  std::size_t p2 = FileName.rfind(key1);
  std::size_t p1 = FileName.rfind(key2);
  if (p2 == std::string::npos)
    p2 = 0;
  if (p1 == std::string::npos)
    p1 = 0;

  return FileName.substr(p1, p2 - p1);
}

std::string Generator::ReplaceAll(std::string str,
                                  const std::string& from,
                                  const std::string& to) {
  std::size_t pos = 0;
  while ((pos = str.find(from, pos)) != std::string::npos) {
    str.replace(pos, from.length(), to);
    pos += to.length();
  }
  return str;
}

bool Generator::IsDelegateType(const BaseType& type) {
  for (auto& i : GetDocument().GetBlocks()) {
    if (i->GetType() != Block::TYPE_INTERFACE)
      continue;
    Interface& iface = static_cast<Interface&>(*i);

    if (IsDelegateType(iface, type))
      return true;
  }

  return false;
}

bool Generator::IsDelegateType(const Interface& inf,
                               const BaseType& type) {
  for (auto& i : inf.GetDeclarations().GetDecls()) {
    if (i->GetMethodType() != Declaration::MethodType::DELEGATE)
      continue;
    if (i->GetID() == type.ToString())
      return true;
  }
  return false;
}

}  // namespace tidl
